@@ -15,7 +15,7 @@
 #
 
 package MongoDB::Connection;
-our $VERSION = '0.42';
+our $VERSION = '0.43';
 
 # ABSTRACT: A connection to a Mongo server
 
@@ -34,9 +34,9 @@ MongoDB::Connection - A connection to a Mongo server
 
 =head1 SYNOPSIS
 
-The MongoDB::Connection class creates a connection to the MongoDB server. 
+The MongoDB::Connection class creates a connection to the MongoDB server.
 
-By default, it connects to a single server running on the local machine 
+By default, it connects to a single server running on the local machine
 listening on the default port:
 
     # connects to localhost:27017
@@ -50,7 +50,7 @@ See the L</"host"> section for more options for connecting to MongoDB.
 
 =head2 Multithreading
 
-Cloning instances of this class is disabled in Perl 5.8.7+, so forked threads 
+Cloning instances of this class is disabled in Perl 5.8.7+, so forked threads
 will have to create their own connections to the database.
 
 =head1 SEE ALSO
@@ -61,7 +61,7 @@ Core documentation on connections: L<http://dochub.mongodb.org/core/connections>
 
 =head2 host
 
-Server or servers to connect to. Defaults to C<mongodb://localhost:27017>.  
+Server or servers to connect to. Defaults to C<mongodb://localhost:27017>.
 
 To connect to more than one database server, use the format:
 
@@ -69,26 +69,26 @@ To connect to more than one database server, use the format:
 
 An arbitrary number of hosts can be specified.
 
-The connect method will return success if it can connect to at least one of the 
-hosts listed.  If it cannot connect to any hosts, it will die. 
+The connect method will return success if it can connect to at least one of the
+hosts listed.  If it cannot connect to any hosts, it will die.
 
-If a port is not specified for a given host, it will default to 27017. For 
+If a port is not specified for a given host, it will default to 27017. For
 example, to connecting to C<localhost:27017> and C<localhost:27018>:
 
     $conn = MongoDB::Connection->new("host" => "mongodb://localhost,localhost:27018");
 
 This will succeed if either C<localhost:27017> or C<localhost:27018> are available.
 
-The connect method will also try to determine who is master if more than one 
+The connect method will also try to determine who is master if more than one
 server is given.  It will try the hosts in order from left to right.  As soon as
-one of the hosts reports that it is master, the connect will return success.  If 
-no hosts report themselves as masters, the connect will die, reporting that it 
+one of the hosts reports that it is master, the connect will return success.  If
+no hosts report themselves as masters, the connect will die, reporting that it
 could not find a master.
 
-If username and password are given, success is conditional on being able to log 
+If username and password are given, success is conditional on being able to log
 into the database as well as connect.  By default, the driver will attempt to
 authenticate with the admin database.  If a different database is specified
-using the C<db_name> property, it will be used instead.  
+using the C<db_name> property, it will be used instead.
 
 =cut
 
@@ -103,20 +103,20 @@ has host => (
 
 I<Only supported in MongoDB server version 1.5+.>
 
-The default number of mongod slaves to replicate a change to before reporting 
+The default number of mongod slaves to replicate a change to before reporting
 success for all operations on this collection.
 
 Defaults to 1 (just the current master).
 
-If this is not set, a safe insert will wait for 1 machine (the master) to 
-ack the operation, then return that it was successful.  If the master has 
+If this is not set, a safe insert will wait for 1 machine (the master) to
+ack the operation, then return that it was successful.  If the master has
 slaves, the slaves may not yet have a record of the operation when success is
-reported.  Thus, if the master goes down, the slaves will never get this 
-operation.  
+reported.  Thus, if the master goes down, the slaves will never get this
+operation.
 
 To prevent this, you can set C<w> to a value greater than 1.  If you set C<w> to
 <N>, it means that safe operations must have succeeded on the master and C<N-1>
-slaves before the client is notified that the operation succeeded.  If the 
+slaves before the client is notified that the operation succeeded.  If the
 operation did not succeed or could not be replicated to C<N-1> slaves within the
 timeout (see C<wtimeout> below), the safe operation will fail (croak).
 
@@ -124,14 +124,14 @@ Some examples of a safe insert with C<w> set to 3 and C<wtimeout> set to 100:
 
 =over 4
 
-=item The master inserts the document, but 100 milliseconds pass before the 
+=item The master inserts the document, but 100 milliseconds pass before the
 slaves have a chance to replicate it.  The master returns failure and the client
 croaks.
 
-=item The master inserts the document and two or more slaves replicate the 
+=item The master inserts the document and two or more slaves replicate the
 operation within 100 milliseconds.  The safe insert returns success.
 
-=item The master inserts the document but there is only one slave up.  The 
+=item The master inserts the document but there is only one slave up.  The
 safe insert times out and croaks.
 
 =back
@@ -146,7 +146,7 @@ has w => (
 
 =head2 wtimeout
 
-The number of milliseconds an operation should wait for C<w> slaves to replicate 
+The number of milliseconds an operation should wait for C<w> slaves to replicate
 it.
 
 Defaults to 1000 (1 second).
@@ -233,7 +233,7 @@ has timeout => (
 
 =head2 username
 
-Username for this connection.  Optional.  If this and the password field are 
+Username for this connection.  Optional.  If this and the password field are
 set, the connection will attempt to authenticate on connection/reconnection.
 
 =cut
@@ -246,7 +246,7 @@ has username => (
 
 =head2 password
 
-Password for this connection.  Optional.  If this and the username field are 
+Password for this connection.  Optional.  If this and the username field are
 set, the connection will attempt to authenticate on connection/reconnection.
 
 =cut
@@ -259,8 +259,8 @@ has password => (
 
 =head2 db_name
 
-Database to authenticate on for this connection.  Optional.  If this, the 
-username, and the password fields are set, the connection will attempt to 
+Database to authenticate on for this connection.  Optional.  If this, the
+username, and the password fields are set, the connection will attempt to
 authenticate against this database on connection/reconnection.  Defaults to
 "admin".
 
@@ -284,14 +284,14 @@ has db_name => (
 This will cause all queries (including C<find_one>s and C<run_command>s) to die
 after this period if the database has not responded.
 
-This value is in milliseconds and defaults to the value of 
+This value is in milliseconds and defaults to the value of
 L<MongoDB::Cursor/timeout>.
 
     $MongoDB::Cursor::timeout = 5000;
     # query timeout for $conn will be 5 seconds
     my $conn = MongoDB::Connection->new;
 
-A value of -1 will cause the driver to wait forever for responses and 0 will 
+A value of -1 will cause the driver to wait forever for responses and 0 will
 cause it to die immediately.
 
 This value overrides L<MongoDB::Cursor/timeout>.
@@ -309,9 +309,23 @@ has query_timeout => (
     default  => sub { return $MongoDB::Cursor::timeout; },
 );
 
+=head2 max_bson_size
+
+This is the largest document, in bytes, storable by MongoDB. The driver queries
+MongoDB on connection to determine this value.  It defaults to 4MB.
+
+=cut
+
+has max_bson_size => (
+    is       => 'ro',
+    isa      => 'Int',
+    lazy     => 1,
+    builder  => '_get_max_bson_size'
+);
+
 =head2 find_master
 
-If this is true, the driver will attempt to find a master given the list of 
+If this is true, the driver will attempt to find a master given the list of
 hosts.  The master-finding algorithm looks like:
 
     for host in hosts
@@ -323,13 +337,13 @@ hosts.  The master-finding algorithm looks like:
             master := replica set's master
             return master
 
-If no master is found, the connection will fail. 
+If no master is found, the connection will fail.
 
 If this is not set (or set to the default, 0), the driver will simply use the
-first host in the host list for all connections.  This can be useful for 
+first host in the host list for all connections.  This can be useful for
 directly connecting to slaves for reads.
 
-If you are connecting to a slave, you should check out the 
+If you are connecting to a slave, you should check out the
 L<MongoDB::Cursor/slave_okay> documentation for information on reading from a
 slave.
 
@@ -495,6 +509,17 @@ sub BUILD {
     $self->_init_conn_holder($master);
 }
 
+sub _get_max_bson_size {
+    my $self = shift;
+    my $buildinfo = $self->get_database('admin')->run_command({buildinfo => 1});
+    my $max = $buildinfo->{'maxBsonObjectSize'};
+    if ($max) {
+        return $max;
+    }
+    # default: 4MB
+    return 4194304;
+}
+
 =head1 METHODS
 
 =head2 connect
@@ -571,7 +596,7 @@ sub _get_any_connection {
     $master = $connection->get_master
 
 Determines which host of a paired connection is master.  Does nothing for
-a non-paired connection.  This need never be invoked by a user, it is 
+a non-paired connection.  This need never be invoked by a user, it is
 called automatically by internal functions.  Returns the index of the master
 connection in the list of connections or -1 if it cannot be determined.
 
@@ -650,7 +675,7 @@ sub _old_stupid_paired_conn {
     };
     if (!($@ =~ m/couldn't connect to server/)) {
         $master = $left->find_one('admin.$cmd', {ismaster => 1});
-        if ($master->{'ismaster'}) {    
+        if ($master->{'ismaster'}) {
             return 0;
         }
     }
@@ -679,7 +704,7 @@ and C<$password>. Passwords are expected to be cleartext and will be
 automatically hashed before sending over the wire, unless C<$is_digest> is
 true, which will assume you already did the hashing on yourself.
 
-See also the core documentation on authentication: 
+See also the core documentation on authentication:
 L<http://dochub.mongodb.org/core/authentication>.
 
 =cut
@@ -706,7 +731,7 @@ sub authenticate {
     # run the login command
     my $login = tie(my %hash, 'Tie::IxHash');
     %hash = (authenticate => 1,
-             user => $username, 
+             user => $username,
              nonce => $nonce,
              key => $digest);
     $result = $db->run_command($login);
@@ -719,7 +744,7 @@ sub authenticate {
     my ($insert, $ids) = MongoDB::write_insert('foo.bar', [{name => "joe", age => 40}]);
     $conn->send($insert);
 
-Low-level function to send a string directly to the database.  Use 
+Low-level function to send a string directly to the database.  Use
 L<MongoDB::write_insert>, L<MongoDB::write_update>, L<MongoDB::write_remove>, or
 L<MongoDB::write_query> to create a valid string.
 
@@ -727,9 +752,9 @@ L<MongoDB::write_query> to create a valid string.
 
     my $cursor = $conn->recv({ns => "foo.bar"});
 
-Low-level function to receive a response from the database. Returns a 
-C<MongoDB::Cursor>.  At the moment, the only required field for C<$info> is 
-"ns", although "request_id" is likely to be required in the future.  The 
+Low-level function to receive a response from the database. Returns a
+C<MongoDB::Cursor>.  At the moment, the only required field for C<$info> is
+"ns", although "request_id" is likely to be required in the future.  The
 C<$info> hash will be automatically created for you by L<MongoDB::write_query>.
 
 =cut
